#import "WordCounterAppDelegate.h"

@implementation WordCounterAppDelegate

@synthesize window = _window;
@synthesize wordsView = _wordsView;
@synthesize countLabel = _countLabel;
@synthesize uniqueLabel = _uniqueLabel;
@synthesize countButton = _countButton;
@synthesize spinner = _spinner;


// Return two values from a function.
typedef struct {
    NSUInteger count;
    NSCountedSet *frequencies;
} CountAndSet;


- (CountAndSet) countWords: (NSString *) string {
    NSScanner *scanner = [NSScanner scannerWithString: string];
    NSCharacterSet *whiteSpace = [NSCharacterSet whitespaceAndNewlineCharacterSet];

    NSUInteger count = 0;
    NSCountedSet *wordSet = [NSCountedSet set];

    NSString *word;
    while ([scanner scanUpToCharactersFromSet: whiteSpace  intoString: &word]) {
        [wordSet addObject: [word lowercaseString]];
        count++;
    }

    CountAndSet result = { count, wordSet };

    sleep (2); // Machines these days are too dang fast!

    return result;

} // countWords


#define GCD_COUNT 1

#if GCD_COUNT

- (void) disableEverything {
    // NSTextView doesn't have an enabled binding, so we'll just do it all here.
    [self.wordsView setEditable: NO];
    [self.countButton setEnabled: NO];
    [self.spinner setHidden: NO];
    [self.spinner startAnimation: self];
} // disableEverything


- (void) enableEverything {
    [self.wordsView setEditable: YES];
    [self.countButton setEnabled: YES];
    [self.spinner stopAnimation: self];
    [self.spinner setHidden: YES];
} // enableEverything


#if 1
- (IBAction) count: (id) sender {

    [self disableEverything];

    dispatch_async (dispatch_get_global_queue (DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            CountAndSet result = [self countWords: self.wordsView.string];
            
            // The set is autoreleased, but we want it to survive until the
            // next block runs, at some unspecified point in the future.
            [result.frequencies retain];
            
            dispatch_async (dispatch_get_main_queue(), ^{
                    NSString *labelString =
                        [NSString stringWithFormat: @"Word count: %u", result.count];
                    [self.countLabel setStringValue: labelString];
                    
                    labelString =
                        [NSString stringWithFormat: @"Unique words: %u",
                                  result.frequencies.count];

                    [self.uniqueLabel setStringValue: labelString];
                    [self enableEverything];

                    [result.frequencies release];

                });
        });

} // count
#endif


#if 0

- (IBAction) count: (id) sender {

    [self disableEverything];

    CountAndSet result = [self countWords: self.wordsView.string];
    
    // The set is autoreleased, but we want it to survive until the
    // next block runs, at some unspecified point in the future.
    [result.frequencies retain];
    
    NSString *labelString =
        [NSString stringWithFormat: @"Word count: %u", result.count];
    [self.countLabel setStringValue: labelString];
    
    labelString =
        [NSString stringWithFormat: @"Unique words: %u",
                  result.frequencies.count];
    
    [self.uniqueLabel setStringValue: labelString];
    [self enableEverything];
    
    [result.frequencies release];


} // count

#endif

#else

- (IBAction) count: (id) sender {
    CountAndSet result = [self countWords: self.wordsView.string];

    NSString *labelString = [NSString stringWithFormat: @"Word count: %u", result.count];
    [self.countLabel setStringValue: labelString];

    labelString = [NSString stringWithFormat: @"Unique words: %u", 
                            result.frequencies.count];
    [self.uniqueLabel setStringValue: labelString];

} // count


#endif


@end // WordCounterAppDelegate

